#include <vector>
#include <istream>
#include <fstream>
#include <sstream>
#include <string>
#include <iomanip>

#include "LMHS_CPP_API.h"
#include "GlobalConfig.h"
#include "Solver.h"
#include "ProblemInstance.h"
//#include "CoprocessorInterface.h"
#include "VarMapper.h"
#include "Util.h"
#include "Timer.h"

using namespace std;

namespace LMHS {

Solver * solver;
ProblemInstance * instance;
ofstream nullstream("/dev/null");
//CoprocessorInterface *ci;
VarMapper vm;
bool preprocessed;

void reset() { 
  delete solver;
  delete instance;
}

void setArgs(int argc, const char* argv[]) {
  GlobalConfig::get().parseArgs(argc, argv, nullstream);
}
/*
void preprocess() {
  
  ci = new CoprocessorInterface();

  condTerminate(preprocessed, 1, "Instance already preprocessed");
  preprocessed = true;

  GlobalConfig & cfg = GlobalConfig::get();

  for (auto core : solver->cores) {
    vector<int> coreClause;
    for (auto lit : core)
      coreClause.push_back(lit);
    solver->instance.addSoftClauseWithBv(coreClause);
  }

  // write the original instance to string buffer
  stringstream orig_instance;
  orig_instance << setprecision(cfg.streamPrecision);
  instance->toLCNF(orig_instance);

  // delete existing instance and solver
  reset();

  // map variables to range 1..n
  stringstream mapped;
  mapped << setprecision(cfg.streamPrecision);
  vm.map(orig_instance, mapped);

  // preprocess mapped instance    
  stringstream pre;
  pre << setprecision(cfg.streamPrecision);
  int ci_status = ci->CP_preprocess(mapped, pre);

  // check for anomalies
  if (ci_status == CI_UNSAT) {
    printf("s UNSATISFIABLE\n");
    exit(1);
  } else if (ci_status == CI_SAT) {
    // todo: 
    stringstream model;
    model << setprecision(cfg.streamPrecision);
    ci->CP_completeModel(pre, model);
    vm.unmap(model, cout);
    exit(1);
  } else if (ci_status == CI_ERR) {
    printf("c coprocessor error\n");
    printf("s UNKNOWN\n");
    exit(1);
  } // else ci_status == CI_UNKNOWN

  // initialize a new instance and solver with preprocessed instance
  initialize(pre);  
}
*/
bool initialize(istream & wcnf_in) {
  instance = new ProblemInstance(wcnf_in, nullstream);
  solver = new Solver(*instance, nullstream);
  return true;
}

bool initialize() {
  instance = new ProblemInstance(nullstream);
  solver = new Solver(*instance, nullstream);
  return true;
}

bool initialize(weight_t top, vector<weight_t>& weights, vector<vector<int>>& clauses) {
  instance = new ProblemInstance(nullstream);

  int max_orig_var = 0;
  for (auto cl : clauses)
    for (auto l : cl)
      max_orig_var = max(max_orig_var, abs(l));
  instance->max_var = max_orig_var;

  for (unsigned i = 0; i < clauses.size(); ++i) {
    if (weights[i] == top) {
      instance->addHardClause(clauses[i]);
    } else {
      instance->addSoftClause(clauses[i], weights[i]);
    }
  }
  solver = new Solver(*instance, nullstream);
  return true;
}

int getNewVariable() {
  return instance->sat_solver->newVar();
}

void addHardClause(vector<int>& lits) {
  solver->instance.addHardClause(lits);
}

int addSoftClause(weight_t weight, vector<int>& lits) {
  int bvar = solver->instance.addSoftClause(lits, weight);
  return bvar;
}

void addSoftClauseWithBv(vector<int>& lits) {
  solver->instance.addSoftClauseWithBv(lits);
}

void addCoreConstraint(vector<int>& core) {
  solver->processCore(core);
}

bool getSolution(weight_t & out_weight, vector<int> & out_solution) {
  out_weight = -1;
  out_solution.clear();

  //GlobalConfig & cfg = GlobalConfig::get();

  if (solver->hardClausesSatisfiable()) {
    solver->solveMaxHS();
    /*if (preprocessed) {
      // get model from solver
      stringstream internal_model;
      internal_model << setprecision(cfg.streamPrecision);
      instance->printSolution(internal_model);

      // reconstruct model from preprocessing trace
      stringstream prepro_model;
      prepro_model << setprecision(cfg.streamPrecision);
      ci->CP_completeModel(internal_model, prepro_model);

      // unmap variables from 1..n to their original values
      stringstream unmapped_model;
      unmapped_model << setprecision(cfg.streamPrecision);
      vm.unmap(prepro_model, unmapped_model);

      string str;
      bool has_objective = false,
           has_model = false;

      while (std::getline(unmapped_model, str)) {
        stringstream linestream(str);

        char c = linestream.get();

        if (c == 'o') {
          has_objective = true;
          linestream >> out_weight;
        }

        if (c == 'v') {
          has_model = true;
          while (true) {
            int lit;
            linestream >> lit;
            if (!linestream) break;
            out_solution.push_back(lit);
          }
        }
      }

      return (has_objective && has_model);
*/
  //  } else {
      out_solution = instance->UB_solution;
      out_weight = instance->UB;  
  //  }
  }
  return out_solution.size() != 0;
}

void forbidLastHS() {
  solver->instance.forbidCurrentMIPSol();
}

void forbidLastModel() {
  solver->instance.forbidCurrentModel();
}

void printStats() { solver->printStats(); }

void declareBvar(int var, weight_t weight, bool pol) {
  solver->instance.addBvar(var, weight);
  if (!pol)
    solver->instance.flippedInternalVarPolarity[abs(var)] = true;
}

}